#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_ssids_statuses_by_device_info
short_description: Information module for organizations _wireless _ssids _statuses
  _by _device
description:
  - Get all organizations _wireless _ssids _statuses _by _device.
  - List status information of all BSSIDs in your organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  networkIds:
    description:
      - NetworkIds query parameter. Optional parameter to filter the result set by
        the included set of network IDs.
    elements: str
    type: list
  serials:
    description:
      - >
        Serials query parameter. A list of serial numbers. The returned devices will
        be filtered to only include these serials.
    elements: str
    type: list
  bssids:
    description:
      - Bssids query parameter. A list of BSSIDs. The returned devices will be filtered
        to only include these BSSIDs.
    elements: str
    type: list
  hideDisabled:
    description:
      - HideDisabled query parameter. If true, the returned devices will not include
        disabled SSIDs. (default true).
    type: bool
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 500. Default is 100.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless getOrganizationWirelessSsidsStatusesByDevice
    description: Complete reference of the getOrganizationWirelessSsidsStatusesByDevice
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-wireless-ssids-statuses-by-device
notes:
  - SDK Method used are
    wireless.Wireless.get_organization_wireless_ssids_statuses_by_device,
  - Paths used are
    get /organizations/{organizationId}/wireless/ssids/statuses/byDevice,
"""

EXAMPLES = r"""
- name: Get all organizations _wireless _ssids _statuses _by _device
  cisco.meraki.organizations_wireless_ssids_statuses_by_device_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkIds: []
    serials: []
    bssids: []
    hideDisabled: true
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "serial": "string",
          "name": "string",
          "network": {
            "id": "string",
            "name": "string"
          },
          "basicServiceSets": [
            {
              "bssid": "string",
              "ssid": {
                "name": "string",
                "number": 0,
                "enabled": true,
                "advertised": true
              },
              "radio": {
                "band": "string",
                "channel": 0,
                "channelWidth": 0,
                "power": 0,
                "isBroadcasting": true,
                "index": "string"
              }
            }
          ]
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_capacityreservationgroup
version_added: "2.4.0"
short_description: Manage Capacity Reservation Group
description:
    - Create, update or delete the Capacity Reservation Group.
options:
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    name:
        description:
            - The name of the Capacity Reservation Group.
        required: true
        type: str
    zones:
        description:
            - Availability Zones to use for this capacity reservation group.
            - The zones can be assigned only during creation.
            - If not provided, the group supports only regional resources in the region.
            - If provided, enforces each capacity reservation in the group to be in one of the zones.
        type: list
        elements: str
        choices:
            - '1'
            - '2'
            - '3'
    state:
        description:
            - State of the Capacity Reservation Group. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Create a Capacity Reservation Group
  azure_rm_capacityreservationgroup:
    resource_group: myResourceGroup
    name: testname01
    zones:
      - 1
      - 2
    tags:
      key: value

- name: Delete the Capacity Reservation Group
  azure_rm_capacityreservationgroup:
    resource_group: myResourceGroup
    name: testname01
    state: absent
'''
RETURN = '''
state:
    description:
        - Current state of the Capacity Reservation Group.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Compute/capacityReservationGroups/testname01"
        location:
            description:
                - The Geo-location where the resource lives.
            returned: always
            type: str
            sample: eastus
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: testname01
        resource_group:
            description:
                - Name of resource group.
            type: str
            returned: always
            sample: myResourceGroup
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: { 'key': 'value' }
        zones:
            description:
                -  A list of all capacity reservation resource ids that belong to capacity reservation group.
            returned: always
            type: list
            sample: ['1', '2']
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMCapacityReservationGroup(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            location=dict(type='str'),
            zones=dict(type='list', elements='str', choices=['1', '2', '3']),
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.location = None
        self.zones = None

        self.body = dict()

        self.results = dict(
            changed=False,
            state=dict()
        )

        super(AzureRMCapacityReservationGroup, self).__init__(self.module_arg_spec,
                                                              supports_tags=True,
                                                              supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])
            for key in ['tags', 'zones']:
                self.body[key] = kwargs[key]

        old_response = self.get_by_name()

        changed = False
        results = None

        if old_response is not None:
            if self.state == 'present':
                update_tags, self.body['tags'] = self.update_tags(old_response['tags'])
                if self.location is not None and self.location.lower() != old_response.get('location').lower():
                    self.fail('The parameters location not support to udpate')
                elif self.body.get('zones') is not None and not all(key in old_response['zones'] for key in self.body[key]):
                    self.fail('The parameters zones not support to udpate')
                elif update_tags:
                    changed = True
                    if not self.check_mode:
                        results = self.update_capacity_reservation_groups(dict(tags=self.body['tags']))
                else:
                    results = old_response
            else:
                changed = True
                if not self.check_mode:
                    results = self.delete_capacity_reservation_groups()
        else:
            resource_group = self.get_resource_group(self.resource_group)
            if self.location is None:
                # Set default location
                self.body['location'] = resource_group.location

            if self.state == 'present':
                changed = True
                if not self.check_mode:
                    results = self.create_capacity_reservation_groups(self.body)
            else:
                changed = False
                self.log("The Capacity Reservation Group is not exists")

        self.results['changed'] = changed
        self.results['state'] = results

        return self.results

    def get_by_name(self):
        response = None
        try:
            response = self.compute_client.capacity_reservation_groups.get(self.resource_group, self.name)

        except ResourceNotFoundError as exec:
            self.log("Failed to get ssh public keys, Exception as {0}".format(exec))

        return self.to_dict(response)

    def create_capacity_reservation_groups(self, body):
        response = None
        try:
            response = self.to_dict(self.compute_client.capacity_reservation_groups.create_or_update(self.resource_group, self.name, body))
        except Exception as exc:
            self.fail("Error creating Capacity Reservation Group {0} - {1}".format(self.name, str(exc)))

        return self.to_dict(response)

    def update_capacity_reservation_groups(self, body):
        response = None
        try:
            response = self.compute_client.capacity_reservation_groups.update(self.resource_group, self.name, body)
        except Exception as exc:
            self.fail("Error updating Capacity Reservation Group {0} - {1}".format(self.name, str(exc)))
        return self.to_dict(response)

    def delete_capacity_reservation_groups(self):
        try:
            self.compute_client.capacity_reservation_groups.delete(self.resource_group, self.name)
        except Exception as exc:
            if self.get_by_name() is not None:
                self.fail("Error deleting Capacity Reservation Group {0} - {1}".format(self.name, str(exc)))

    def to_dict(self, body):
        if body is None:
            results = None
        elif isinstance(body, dict):
            results = dict(
                resource_group=self.resource_group,
                id=body.get('id'),
                name=body.get('name'),
                location=body.get('location'),
                zones=body.get('zones'),
                tags=body.get('tags'))
        else:
            results = dict(
                resource_group=self.resource_group,
                id=body.id,
                name=body.name,
                location=body.location,
                zones=body.zones,
                tags=body.tags)
        return results


def main():
    AzureRMCapacityReservationGroup()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2015, René Moser <mail@renemoser.net>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ipify_facts
short_description: Retrieve the public IP of your internet gateway
description:
  - If behind NAT and need to know the public IP of your internet gateway.
author:
  - René Moser (@resmo)
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.facts
  - community.general._attributes.facts_module
options:
  api_url:
    description:
      - URL of the ipify.org API service.
      - C(?format=json) is appended by default.
    type: str
    default: https://api.ipify.org/
  timeout:
    description:
      - HTTP connection timeout in seconds.
    type: int
    default: 10
  validate_certs:
    description:
      - When set to V(false), SSL certificates are not validated.
    type: bool
    default: true
notes:
  - Visit https://www.ipify.org to get more information.
"""

EXAMPLES = r"""
# Gather IP facts from ipify.org
- name: Get my public IP
  community.general.ipify_facts:

# Gather IP facts from your own ipify service endpoint with a custom timeout
- name: Get my public IP
  community.general.ipify_facts:
    api_url: http://api.example.com/ipify
    timeout: 20
"""

RETURN = r"""
ansible_facts:
  description: The returned facts.
  returned: success
  type: dict
  contains:
    ipify_public_ip:
      description: Public IP of the internet gateway.
      returned: success
      type: str
      sample: 1.2.3.4
"""

import json

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_text
from ansible.module_utils.urls import fetch_url


class IpifyFacts:
    def __init__(self):
        self.api_url = module.params.get("api_url")
        self.timeout = module.params.get("timeout")

    def run(self):
        result = {"ipify_public_ip": None}
        (response, info) = fetch_url(module=module, url=f"{self.api_url}?format=json", force=True, timeout=self.timeout)

        if not response:
            module.fail_json(
                msg=f"No valid or no response from url {self.api_url} within {self.timeout} seconds (timeout)"
            )

        data = json.loads(to_text(response.read()))
        result["ipify_public_ip"] = data.get("ip")
        return result


def main():
    global module
    module = AnsibleModule(
        argument_spec=dict(
            api_url=dict(type="str", default="https://api.ipify.org/"),
            timeout=dict(type="int", default=10),
            validate_certs=dict(type="bool", default=True),
        ),
        supports_check_mode=True,
    )

    ipify_facts = IpifyFacts().run()
    ipify_facts_result = dict(changed=False, ansible_facts=ipify_facts)
    module.exit_json(**ipify_facts_result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Vrf_global parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Vrf_globalTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Vrf_globalTemplate, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "name",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "vrf {{ name }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                },
            },
            "shared": True,
        },
        {
            "name": "description",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                \s+description\s(?P<description>.+$)
                $""", re.VERBOSE,
            ),
            "setval": "description {{ description }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    'description': '{{ description }}',
                },
            },
        },
        {
            "name": "evpn_route_sync",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                \s+evpn-route-sync\s(?P<evpn_route_sync>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "evpn-route-sync {{ evpn_route_sync }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "evpn_route_sync": "{{ evpn_route_sync }}",
                },
            },
        },
        {
            "name": "fallback_vrf",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                \s+fallback-vrf\s(?P<fallback_vrf>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "fallback-vrf {{ fallback_vrf }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "fallback_vrf": "{{ fallback_vrf }}",
                },
            },
        },
        {
            "name": "mhost.default_interface",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<mhost>\s+mhost\s(?P<afi>\S+))
                \s+default-interface\s(?P<default_interface>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "mhost {{ mhost.afi }} default-interface {{ mhost.default_interface }}",
            "compval": "mhost.default_interface",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "mhost": {
                        "afi": "{{ afi }}",
                        "default_interface": "{{ default_interface }}",
                    },
                },
            },
        },
        {
            "name": "rd",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                \s+rd\s(?P<rd>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "rd {{ rd }}",
            "compval": "rd",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "rd": "{{ rd }}",
                },
            },
        },
        {
            "name": "remote_route_filtering.disable",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                \s+remote-route-filtering\s(?P<disable>disable)
                $""", re.VERBOSE,
            ),
            "setval": "remote-route-filtering disable",
            "compval": "remote_route_filtering.disable",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "remote_route_filtering": {
                        "disable": "{{ true if disable is defined }}",
                    },
                },
            },
        },
        {
            "name": "vpn.id",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                \s+vpn\sid\s(?P<vpn_id>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "vpn id {{ vpn.id }}",
            "compval": "vpn.id",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "vpn": {
                        "id": "{{ vpn_id }}",
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
#
# Copyright (c) 2015, Brian Coca <bcoca@ansible.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: jabber
short_description: Send a message to jabber user or chat room
description:
  - Send a message to jabber.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  user:
    type: str
    description:
      - User as which to connect.
    required: true
  password:
    type: str
    description:
      - Password for user to connect.
    required: true
  to:
    type: str
    description:
      - User ID or name of the room, when using room use a slash to indicate your nick.
    required: true
  msg:
    type: str
    description:
      - The message body.
    required: true
  host:
    type: str
    description:
      - Host to connect, overrides user info.
  port:
    type: int
    description:
      - Port to connect to, overrides default.
    default: 5222
  encoding:
    type: str
    description:
      - Message encoding.
requirements:
  - python xmpp (xmpppy)
author: "Brian Coca (@bcoca)"
"""

EXAMPLES = r"""
- name: Send a message to a user
  community.general.jabber:
    user: mybot@example.net
    password: secret
    to: friend@example.net
    msg: Ansible task finished

- name: Send a message to a room
  community.general.jabber:
    user: mybot@example.net
    password: secret
    to: mychaps@conference.example.net/ansiblebot
    msg: Ansible task finished

- name: Send a message, specifying the host and port
  community.general.jabber:
    user: mybot@example.net
    host: talk.example.net
    port: 5223
    password: secret
    to: mychaps@example.net
    msg: Ansible task finished
"""

import time
import traceback

HAS_XMPP = True
XMPP_IMP_ERR = None
try:
    import xmpp
except ImportError:
    XMPP_IMP_ERR = traceback.format_exc()
    HAS_XMPP = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib


def main():
    module = AnsibleModule(
        argument_spec=dict(
            user=dict(required=True),
            password=dict(required=True, no_log=True),
            to=dict(required=True),
            msg=dict(required=True),
            host=dict(),
            port=dict(default=5222, type="int"),
            encoding=dict(),
        ),
        supports_check_mode=True,
    )

    if not HAS_XMPP:
        module.fail_json(msg=missing_required_lib("xmpppy"), exception=XMPP_IMP_ERR)

    jid = xmpp.JID(module.params["user"])
    user = jid.getNode()
    server = jid.getDomain()
    port = module.params["port"]
    password = module.params["password"]
    try:
        to, nick = module.params["to"].split("/", 1)
    except ValueError:
        to, nick = module.params["to"], None

    if module.params["host"]:
        host = module.params["host"]
    else:
        host = server
    if module.params["encoding"]:
        xmpp.simplexml.ENCODING = module.params["encoding"]

    msg = xmpp.protocol.Message(body=module.params["msg"])

    try:
        conn = xmpp.Client(server, debug=[])
        if not conn.connect(server=(host, port)):
            module.fail_json(rc=1, msg=f"Failed to connect to server: {server}")
        if not conn.auth(user, password, "Ansible"):
            module.fail_json(rc=1, msg=f"Failed to authorize {user} on: {server}")
        # some old servers require this, also the sleep following send
        conn.sendInitPresence(requestRoster=0)

        if nick:  # sending to room instead of user, need to join
            msg.setType("groupchat")
            msg.setTag("x", namespace="http://jabber.org/protocol/muc#user")
            join = xmpp.Presence(to=module.params["to"])
            join.setTag("x", namespace="http://jabber.org/protocol/muc")
            conn.send(join)
            time.sleep(1)
        else:
            msg.setType("chat")

        msg.setTo(to)
        if not module.check_mode:
            conn.send(msg)
        time.sleep(1)
        conn.disconnect()
    except Exception as e:
        module.fail_json(msg=f"unable to send msg: {e}", exception=traceback.format_exc())

    module.exit_json(changed=False, to=to, user=user, msg=msg.getBody())


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for nxos_ntp_global
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: nxos_ntp_global
short_description: NTP Global resource module.
description:
- This module manages ntp configuration on devices running Cisco NX-OS.
version_added: 2.6.0
notes:
- Tested against NX-OS 9.3.6 on Cisco Nexus Switches.
- This module works with connection C(network_cli) and C(httpapi).
- Tested against Cisco MDS NX-OS 9.2(2) with connection C(network_cli).
author: Nilashish Chakraborty (@NilashishC)
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the NX-OS device
      by executing the command B(show running-config ntp).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A dict of ntp configuration.
    type: dict
    suboptions:
      access_group:
        description:
          - NTP access-group.
          - This option is unsupported on MDS switches.
        type: dict
        suboptions:
          match_all:
            description: Scan ACLs present in all ntp access groups.
            type: bool
          peer:
            description: Access-group peer.
            type: list
            elements: dict
            suboptions:
              access_list:
                description: Name of access list.
                type: str
          query_only:
            description: Access-group query-only.
            type: list
            elements: dict
            suboptions:
              access_list:
                description: Name of access list.
                type: str
          serve:
            description: Access-group serve.
            type: list
            elements: dict
            suboptions:
              access_list:
                description: Name of access list.
                type: str
          serve_only:
            description: Access-group serve-only.
            type: list
            elements: dict
            suboptions:
              access_list:
                description: Name of access list.
                type: str
      allow:
        description: Enable/Disable the packets.
        type: dict
        suboptions:
          control:
            description: Control mode packets.
            type: dict
            suboptions:
              rate_limit:
                description: Rate-limit delay.
                type: int
          private:
            description: Enable/Disable Private mode packets.
            type: bool
      authenticate:
        description: Enable/Disable authentication.
        type: bool
      authentication_keys:
        description: NTP authentication key.
        type: list
        elements: dict
        suboptions:
          id:
            description: Authentication key number (range 1-65535).
            type: int
          key:
            description: Authentication key.
            type: str
          encryption:
            description:
              - 0 for Clear text
              - 7 for Encrypted
            type: int
      logging:
        description: Enable/Disable logging of NTPD Events.
        type: bool
      master:
        description:
          - Act as NTP master clock.
          - This option is unsupported on MDS switches.
        type: dict
        suboptions:
          stratum:
            description: Stratum number.
            type: int
      passive:
        description:
          - NTP passive command.
          - This option is unsupported on MDS switches.
        type: bool
      peers:
        description: NTP Peers.
        type: list
        elements: dict
        suboptions:
          peer:
            description: Hostname/IP address of the NTP Peer.
            type: str
          key_id:
            description: Keyid to be used while communicating to this server.
            type: int
          maxpoll:
            description:
              - Maximum interval to poll a peer.
              - Poll interval in secs to a power of 2.
            type: int
          minpoll:
            description:
              - Minimum interval to poll a peer.
              - Poll interval in secs to a power of 2.
            type: int
          prefer:
            description:
              - Preferred Server.
            type: bool
          vrf:
            description:
              - Display per-VRF information.
              - This option is unsupported on MDS switches.
            type: str
            aliases: ["use_vrf"]
      servers:
        description: NTP servers.
        type: list
        elements: dict
        suboptions:
          server:
            description: Hostname/IP address of the NTP Peer.
            type: str
          key_id:
            description: Keyid to be used while communicating to this server.
            type: int
          maxpoll:
            description:
              - Maximum interval to poll a peer.
              - Poll interval in secs to a power of 2.
            type: int
          minpoll:
            description:
              - Minimum interval to poll a peer.
              - Poll interval in secs to a power of 2.
            type: int
          prefer:
            description:
              - Preferred Server.
            type: bool
          vrf:
            description:
              - Display per-VRF information.
              - This option is not applicable for MDS switches.
            type: str
            aliases: ["use_vrf"]
      source:
        description:
          - Source of NTP packets.
          - This option is unsupported on MDS switches.
        type: str
      source_interface:
        description: Source interface sending NTP packets.
        type: str
      trusted_keys:
        description: NTP trusted-key number.
        type: list
        elements: dict
        suboptions:
          key_id:
            description: Trusted-Key number.
            type: int
  state:
    description:
    - The state the configuration should be left in.
    - The states I(replaced) and I(overridden) have identical
      behaviour for this module.
    - Please refer to examples for more details.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - parsed
    - gathered
    - rendered
    default: merged
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
# nxos-9k-rdo# show running-config ntp
# nxos-9k-rdo#

- name: Merge the provided configuration with the existing running configuration
  cisco.nxos.nxos_ntp_global: &id001
    config:
      access_group:
        peer:
          - access_list: PeerAcl1
        serve:
          - access_list: ServeAcl1
      authenticate: true
      authentication_keys:
        - id: 1001
          key: vagwwtKfkv
          encryption: 7
        - id: 1002
          key: vagwwtKfkvgthz
          encryption: 7
      logging: true
      master:
        stratum: 2
      peers:
        - peer: 192.0.2.1
          key_id: 1
          maxpoll: 15
          minpoll: 5
          vrf: default
        - peer: 192.0.2.2
          key_id: 2
          prefer: true
          vrf: siteA
      servers:
        - server: 198.51.100.1
          key_id: 2
          vrf: default
        - server: 203.0.113.1
          key_id: 1
          vrf: siteB

# Task output
# -------------
#  before: {}
#
#  commands:
#    - "ntp authenticate"
#    - "ntp logging"
#    - "ntp master 2"
#    - "ntp authentication-keys 1001 md5 vagwwtKfkv 7"
#    - "ntp authentication-keys 1002 md5 vagwwtKfkvgthz 7"
#    - "ntp peer 192.0.2.1 use-vrf default key 1 minpoll 5 maxpoll 15"
#    - "ntp peer 192.0.2.2 prefer use-vrf siteA key 2"
#    - "ntp server 198.51.100.1 use-vrf default key 2"
#    - "ntp server 203.0.113.1 use-vrf siteB key 1"
#    - "ntp access-group peer PeerAcl1"
#    - "ntp access-group serve ServeAcl1"
#
#  after:
#    access_group:
#      peer:
#        - access_list: PeerAcl1
#      serve:
#       - access_list: ServeAcl1
#    authenticate: true
#    authentication_keys:
#      - id: 1001
#        key: vagwwtKfkv
#        encryption: 7
#      - id: 1002
#        key: vagwwtKfkvgthz
#        encryption: 7
#    logging: true
#    master:
#     stratum: 2
#    peers:
#      - peer: 192.0.2.1
#        key_id: 1
#        maxpoll: 15
#        minpoll: 5
#        vrf: default
#      - peer: 192.0.2.2
#        key_id: 2
#        prefer: true
#        vrf: siteA
#    servers:
#      - server: 198.51.100.1
#        key_id: 2
#        vrf: default
#      - server: 203.0.113.1
#        key_id: 1
#        vrf: siteB

# After state:
# ------------
# nxos-9k-rdo# show running-config ntp
# ntp authenticate
# ntp logging
# ntp master 2
# ntp authentication-keys 1001 md5 vagwwtKfkv 7
# ntp authentication-keys 1002 md5 vagwwtKfkvgthz 7
# ntp peer 192.0.2.1 use-vrf default key 1 minpoll 5 maxpoll 15
# ntp peer 192.0.2.2 prefer use-vrf siteA key 2
# ntp server 198.51.100.1 use-vrf default key 2
# ntp server 203.0.113.1 use-vrf siteB key 1
# ntp access-group peer PeerAcl1
# ntp access-group serve ServeAcl1

# Using replaced

# Before state:
# ------------
# nxos-9k-rdo# show running-config ntp
# ntp authenticate
# ntp logging
# ntp master 2
# ntp authentication-keys 1001 md5 vagwwtKfkv 7
# ntp authentication-keys 1002 md5 vagwwtKfkvgthz 7
# ntp peer 192.0.2.1 use-vrf default key 1 minpoll 5 maxpoll 15
# ntp peer 192.0.2.2 prefer use-vrf siteA key 2
# ntp server 198.51.100.1 use-vrf default key 2
# ntp server 203.0.113.1 use-vrf siteB key 1
# ntp access-group peer PeerAcl1
# ntp access-group serve ServeAcl1

- name: Replace logging global configurations of listed logging global with provided configurations
  cisco.nxos.nxos_ntp_global:
    config:
      access_group:
        peer:
          - access_list: PeerAcl2
        serve:
          - access_list: ServeAcl2
      logging: true
      master:
        stratum: 2
      peers:
        - peer: 192.0.2.1
          key_id: 1
          maxpoll: 15
          minpoll: 5
          vrf: default
        - peer: 192.0.2.5
          key_id: 2
          prefer: true
          vrf: siteA
      servers:
        - server: 198.51.100.1
          key_id: 2
          vrf: default
    state: replaced

# Task output
# -------------
#  before:
#    access_group:
#      peer:
#        - access_list: PeerAcl1
#      serve:
#       - access_list: ServeAcl1
#    authenticate: true
#    authentication_keys:
#      - id: 1001
#        key: vagwwtKfkv
#        encryption: 7
#      - id: 1002
#        key: vagwwtKfkvgthz
#        encryption: 7
#    logging: true
#    master:
#     stratum: 2
#    peers:
#      - peer: 192.0.2.1
#        key_id: 1
#        maxpoll: 15
#        minpoll: 5
#        vrf: default
#      - peer: 192.0.2.2
#        key_id: 2
#        prefer: true
#        vrf: siteA
#    servers:
#      - server: 198.51.100.1
#        key_id: 2
#        vrf: default
#      - server: 203.0.113.1
#        key_id: 1
#        vrf: siteB
#
#  commands:
#    - "no ntp authenticate"
#    - "no ntp authentication-keys 1001 md5 vagwwtKfkv 7"
#    - "no ntp authentication-keys 1002 md5 vagwwtKfkvgthz 7"
#    - "ntp peer 192.0.2.5 prefer use-vrf siteA key 2"
#    - "no ntp peer 192.0.2.2 prefer use-vrf siteA key 2"
#    - "no ntp server 203.0.113.1 use-vrf siteB key 1"
#    - "ntp access-group peer PeerAcl2"
#    - "no ntp access-group peer PeerAcl1"
#    - "ntp access-group serve ServeAcl2"
#    - "no ntp access-group serve ServeAcl1"
#
#  after:
#    access_group:
#      peer:
#        - access_list: PeerAcl2
#      serve:
#        - access_list: ServeAcl2
#    logging: true
#    master:
#      stratum: 2
#    peers:
#      - peer: 192.0.2.1
#        key_id: 1
#        maxpoll: 15
#        minpoll: 5
#        vrf: default
#      - peer: 192.0.2.5
#        key_id: 2
#        prefer: true
#        vrf: siteA
#    servers:
#      - server: 198.51.100.1
#        key_id: 2
#        vrf: default

# After state:
# ------------
# nxos-9k-rdo# show running-config ntp
# ntp logging
# ntp master 2
# ntp peer 192.0.2.1 use-vrf default key 1 minpoll 5 maxpoll 15
# ntp peer 192.0.2.5 prefer use-vrf siteA key 2
# ntp server 198.51.100.1 use-vrf default key 2
# ntp access-group peer PeerAcl2
# ntp access-group serve ServeAcl2

# Using deleted to delete all logging configurations

# Before state:
# ------------
# nxos-9k-rdo# show running-config ntp

- name: Delete all logging configuration
  cisco.nxos.nxos_ntp_global:
    state: deleted

# Task output
# -------------
#  before:
#    access_group:
#      peer:
#        - access_list: PeerAcl1
#      serve:
#       - access_list: ServeAcl1
#    authenticate: true
#    authentication_keys:
#      - id: 1001
#        key: vagwwtKfkv
#        encryption: 7
#      - id: 1002
#        key: vagwwtKfkvgthz
#        encryption: 7
#    logging: true
#    master:
#     stratum: 2
#    peers:
#      - peer: 192.0.2.1
#        key_id: 1
#        maxpoll: 15
#        minpoll: 5
#        vrf: default
#      - peer: 192.0.2.2
#        key_id: 2
#        prefer: true
#        vrf: siteA
#    servers:
#      - server: 198.51.100.1
#        key_id: 2
#        vrf: default
#      - server: 203.0.113.1
#        key_id: 1
#        vrf: siteB
#
#  commands:
#    - "no ntp authenticate"
#    - "no ntp logging"
#    - "no ntp master 2"
#    - "no ntp authentication-keys 1001 md5 vagwwtKfkv 7"
#    - "no ntp authentication-keys 1002 md5 vagwwtKfkvgthz 7"
#    - "no ntp peer 192.0.2.1 use-vrf default key 1 minpoll 5 maxpoll 15"
#    - "no ntp peer 192.0.2.2 prefer use-vrf siteA key 2"
#    - "no ntp server 198.51.100.1 use-vrf default key 2"
#    - "no ntp server 203.0.113.1 use-vrf siteB key 1"
#    - "no ntp access-group peer PeerAcl1"
#    - "no ntp access-group serve ServeAcl1"
#
#  after: {}

# After state:
# ------------
# nxos-9k-rdo# show running-config ntp
# nxos-9k-rdo#

# Using rendered

- name: Render platform specific configuration lines with state rendered (without connecting to the device)
  cisco.nxos.nxos_ntp_global:
    config:
      access_group:
        peer:
          - access_list: PeerAcl1
        serve:
          - access_list: ServeAcl1
      authenticate: true
      authentication_keys:
        - id: 1001
          key: vagwwtKfkv
          encryption: 7
        - id: 1002
          key: vagwwtKfkvgthz
          encryption: 7
      logging: true
      master:
        stratum: 2
      peers:
        - peer: 192.0.2.1
          key_id: 1
          maxpoll: 15
          minpoll: 5
          vrf: default
        - peer: 192.0.2.2
          key_id: 2
          prefer: true
          vrf: siteA
      servers:
        - server: 198.51.100.1
          key_id: 2
          vrf: default
        - server: 203.0.113.1
          key_id: 1
          vrf: siteB
    state: rendered

# Task Output (redacted)
# -----------------------
#  rendered:
#    - "ntp authenticate"
#    - "ntp logging"
#    - "ntp master 2"
#    - "ntp authentication-keys 1001 md5 vagwwtKfkv 7"
#    - "ntp authentication-keys 1002 md5 vagwwtKfkvgthz 7"
#    - "ntp peer 192.0.2.1 use-vrf default key 1 minpoll 5 maxpoll 15"
#    - "ntp peer 192.0.2.2 prefer use-vrf siteA key 2"
#    - "ntp server 198.51.100.1 use-vrf default key 2"
#    - "ntp server 203.0.113.1 use-vrf siteB key 1"
#    - "ntp access-group peer PeerAcl1"
#    - "ntp access-group serve ServeAcl1"

# Using parsed

# parsed.cfg
# ------------
# ntp authenticate
# ntp logging
# ntp master 2
# ntp authentication-keys 1001 md5 vagwwtKfkv 7
# ntp authentication-keys 1002 md5 vagwwtKfkvgthz 7
# ntp peer 192.0.2.1 use-vrf default key 1 minpoll 5 maxpoll 15
# ntp peer 192.0.2.2 prefer use-vrf siteA key 2
# ntp server 198.51.100.1 use-vrf default key 2
# ntp server 203.0.113.1 use-vrf siteB key 1
# ntp access-group peer PeerAcl1
# ntp access-group serve ServeAcl1

- name: Parse externally provided ntp configuration
  cisco.nxos.nxos_ntp_global:
    running_config: "{{ lookup('file', './fixtures/parsed.cfg') }}"
    state: parsed

# Task output (redacted)
# -----------------------
# parsed:
#    access_group:
#      peer:
#        - access_list: PeerAcl1
#      serve:
#       - access_list: ServeAcl1
#    authenticate: true
#    authentication_keys:
#      - id: 1001
#        key: vagwwtKfkv
#        encryption: 7
#      - id: 1002
#        key: vagwwtKfkvgthz
#        encryption: 7
#    logging: true
#    master:
#     stratum: 2
#    peers:
#      - peer: 192.0.2.1
#        key_id: 1
#        maxpoll: 15
#        minpoll: 5
#        vrf: default
#      - peer: 192.0.2.2
#        key_id: 2
#        prefer: true
#        vrf: siteA
#    servers:
#      - server: 198.51.100.1
#        key_id: 2
#        vrf: default
#      - server: 203.0.113.1
#        key_id: 1
#        vrf: siteB
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - ntp master stratum 2
    - ntp peer 198.51.100.1 use-vrf test maxpoll 7
    - ntp authentication-key 10 md5 wawyhanx2 7
    - ntp access-group peer PeerAcl1
    - ntp access-group peer PeerAcl2
    - ntp access-group query-only QueryAcl1
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - ntp master stratum 2
    - ntp peer 198.51.100.1 use-vrf test maxpoll 7
    - ntp authentication-key 10 md5 wawyhanx2 7
    - ntp access-group peer PeerAcl1
    - ntp access-group peer PeerAcl2
    - ntp access-group query-only QueryAcl1
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.ntp_global.ntp_global import (
    Ntp_globalArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.ntp_global.ntp_global import (
    Ntp_global,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Ntp_globalArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Ntp_global(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Liu Qingyi, (@smile37773)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_galleryimageversion_info
version_added: "0.1.2"
short_description: Get Azure SIG Image Version info
description:
    - Get info of Azure SIG Image Version.
options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: true
    gallery_name:
        description:
            - The name of the Shared Image Gallery in which the Image Definition resides.
        type: str
        required: true
    gallery_image_name:
        description:
            - The name of the gallery Image Definition in which the Image Version resides.
        type: str
        required: true
    name:
        description:
            - Resource name.
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
author:
    - Liu Qingyi (@smile37773)

'''

EXAMPLES = '''
- name: List gallery image versions in a gallery image definition.
  azure_rm_galleryimageversion_info:
    resource_group: myResourceGroup
    gallery_name: myGallery
    gallery_image_name: myImage
- name: Get a gallery image version.
  azure_rm_galleryimageversion_info:
    resource_group: myResourceGroup
    gallery_name: myGallery
    gallery_image_name: myImage
    name: myVersion
'''

RETURN = '''
versions:
    description:
        A list of dict results where the key is the name of the version and the values are the info for that version.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups
      /myResourceGroup/providers/Microsoft.Compute/galleries/myGallery/images/myImage/versions/myVersion"
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: "myVersion"
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: "eastus"
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: { "tag": "value" }
        publishing_profile:
            description:
                - The publishing profile of a gallery image version.
            type: dict
        provisioning_state:
            description:
                - The current state of the gallery.
            type: str
            sample: "Succeeded"
        storage_profile:
            description:
                - This is the storage profile of a Gallery Image Version.
            type: dict
            sample: {
                    "data_disk_images": [{"host_caching": "None", "lun": 0, "size_in_gb": 128}],
                    "os_disk_image": {"host_caching": "ReadOnly", "size_in_gb": 30, "source": {}},
                    "source": {"id": "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Compute/images/testimagea"},
                    "type": "Microsoft.Compute/galleries/images/versions"
            }
        safety_profile:
            description:
                - This is the safety profile of the Gallery Image Version.
            type: dict
            sample: {
                    "allow_deletion_of_replicated_locations": false,
                    "reported_for_policy_violation": false
                }
'''

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    pass


class AzureRMGalleryImageVersionsInfo(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            gallery_name=dict(
                type='str',
                required=True
            ),
            gallery_image_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            ),
        )

        self.resource_group = None
        self.gallery_name = None
        self.gallery_image_name = None
        self.name = None

        self.results = dict(changed=False)
        super(AzureRMGalleryImageVersionsInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name is not None:
            self.results['versions'] = self.get()
        else:
            self.results['versions'] = self.listbygalleryimage()
        return self.results

    def get(self):
        """ Get a single gallery image version"""
        response = None

        try:
            response = self.image_version_client.gallery_image_versions.get(self.resource_group, self.gallery_name, self.gallery_image_name, self.name)
        except ResourceNotFoundError:
            self.log('Could not get the gallery image verison')

        return self.format_item(response)

    def listbygalleryimage(self):
        response = None
        try:
            response = self.image_version_client.gallery_image_versions.list_by_gallery_image(self.resource_group, self.gallery_name, self.gallery_image_name)
        except Exception:
            self.log('Could not list the gallery image versions')

        return [self.format_item(x) for x in response] if response else []

    def format_item(self, item):
        if item is None:
            return None
        else:
            return item.as_dict()


def main():
    AzureRMGalleryImageVersionsInfo()


if __name__ == '__main__':
    main()



#
# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_text
from ansible.module_utils.connection import Connection

from ansible_collections.arista.eos.plugins.module_utils.network.eos.providers import providers


class NetworkModule(AnsibleModule):
    fail_on_missing_provider = True

    def __init__(self, connection=None, *args, **kwargs):
        super(NetworkModule, self).__init__(*args, **kwargs)

        if connection is None:
            connection = Connection(self._socket_path)

        self.connection = connection

    @property
    def provider(self):
        if not hasattr(self, "_provider"):
            capabilities = self.from_json(self.connection.get_capabilities())

            network_os = capabilities["device_info"]["network_os"]
            network_api = capabilities["network_api"]

            if network_api == "cliconf":
                connection_type = "network_cli"

            cls = providers.get(
                network_os,
                self._name.split(".")[-1],
                connection_type,
            )

            if not cls:
                msg = "unable to find suitable provider for network os %s" % network_os
                if self.fail_on_missing_provider:
                    self.fail_json(msg=msg)
                else:
                    self.warn(msg)

            obj = cls(self.params, self.connection, self.check_mode)

            setattr(self, "_provider", obj)

        return getattr(self, "_provider")

    def get_facts(self, subset=None):
        try:
            self.provider.get_facts(subset)
        except Exception as exc:
            self.fail_json(msg=to_text(exc))

    def edit_config(self, config_filter=None):
        current_config = self.connection.get_config(flags=config_filter)
        try:
            commands = self.provider.edit_config(current_config)
            changed = bool(commands)
            return {"commands": commands, "changed": changed}
        except Exception as exc:
            self.fail_json(msg=to_text(exc))



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_virtualnetworkgatewaynatrule

version_added: "2.4.0"

short_description: Gets or list the specified local network gateway in a resource group

description:
    - Gets or list the specified local network gateway in a resource group.

options:
    resource_group:
        description:
            - The local network gateway's resource group.
        type: str
        required: true
    virtual_network_gateway_name:
        description:
            - The name of the local network gateway.
        type: str
        required: true
    name:
        description:
            - he name of the resource that is unique within a resource group.
        type: str
        required: true
    type_properties_type:
        description:
            - The type of NAT rule for VPN NAT.
        type: str
        choices:
            - Dynamic
            - Static
    mode:
        description:
            - The Source NAT direction of a VPN NAT.
        type: str
        choices:
            - EgressSnat
            - IngressSnat
    ip_configuration_id:
        description:
            - The IP Configuration ID this NAT rule applies to.
        type: str
    external_mappings:
        description:
            - The private IP address external mapping for NAT.
        type: list
        elements: str
    internal_mappings:
        description:
            - The private IP address internal mapping for NAT.
        type: list
        elements: str
    state:
        description:
            - Use C(present) to create or update the virtual network gateway nat rule.
            - Use C(absent) to delete the nat rule.
        type: str
        default: present
        choices:
            - absent
            - present
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create a virtual netowrk nat rule
  azure_rm_virtualnetworkgatewaynatrule:
    resource_group: "{{ resource_group }}"
    virtual_network_gateway_name: "{{ vngname }}"
    name: "{{ natrulename }}"
    type_properties_type: Dynamic
    ip_configuration_id: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Network/virtualNetworkGateways/testRG/ipConfigurations/ipconfig"
    mode: EgressSnat
    internal_mappings:
      - 10.1.0.0/24
    external_mappings:
      - 192.168.1.0/24

- name: Delete the virtual netowrk nat rule
  azure_rm_virtualnetworkgatewaynatrule:
    resource_group: "{{ resource_group }}"
    virtual_network_gateway_name: "{{ vngname }}"
    name: "{{ natrulename }}"
    state: absent
'''

RETURN = '''
state:
    description:
        - Gets the nat rules for a particular virtual network gateway
    returned: always
    type: complex
    contains:
        id:
            description:
                - The resource ID.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Network/virtualNetworkGateways/vng01/natRules/natrule"
        internal_mappings:
            description:
                - The private IP address internal mapping for NAT.
            type: list
            returned: always
            sample: ["10.1.0.0/24"]
        external_mappings:
            description:
                - The private IP address external mapping for NAT.
            type: list
            returned: always
            sample: ["192.168.1.0/24"]
        ip_configuration_id:
            description:
                - he IP Configuration ID this NAT rule applies to.
            type: str
            returned: always
            sample:  "/subscriptions/subid/resourceGroups/rg1/providers/Microsoft.Network/virtualNetworkGateways/gateway1/ipConfigurations/default"
        type_properties_type:
            description:
                - The type of NAT rule for VPN NAT.
            type: str
            returned: always
            sample: Static
        mode:
            description:
                - The Source NAT direction of a VPN NAT.
            type: str
            returned: always
            sample: EgressSnat
        name:
            description:
                - The resource name.
            type: str
            returned: always
            sample: natrule_name
        resource_group:
            description:
                - The resource group name.
            type: str
            returned: always
            sample: testRG
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            type: str
            returned: always
            sample: b5a32693-2e75-49e0-9137-ded19db658d6
        provisioning_state:
            description:
                - The provisioning state of the nat rule resource.
            type: str
            returned: always
            sample: Succeeded
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: Microsoft.Network/virtualNetworkGateways/natRules
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import HttpResponseError
    from azure.core.polling import LROPoller
except Exception:
    # handled in azure_rm_common
    pass


class AzureRMVirtualNetworkNatGateway(AzureRMModuleBase):
    """Utility class to get Azure Kubernetes Service Credentials facts"""

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
            virtual_network_gateway_name=dict(type='str', required=True),
            type_properties_type=dict(type='str', choices=['Dynamic', 'Static']),
            mode=dict(type='str', choices=['EgressSnat', 'IngressSnat']),
            ip_configuration_id=dict(type='str'),
            external_mappings=dict(type='list', elements='str'),
            internal_mappings=dict(type='list', elements='str'),
            state=dict(type='str', default='present', choices=['present', 'absent'])
        )

        self.type_properties_type = None
        self.mode = None
        self.ip_configuration_id = None
        self.external_mappings = None
        self.internal_mappings = None

        self.results = dict(
            changed=False,
            state=[],
        )
        required_if = [('type_properties_type', 'Dynamic', ['ip_configuration_id'])]

        super(AzureRMVirtualNetworkNatGateway, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                              supports_check_mode=True,
                                                              required_if=required_if,
                                                              supports_tags=True,
                                                              facts_module=False)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec):
            setattr(self, key, kwargs[key])

        old_response = self.get_nat_rule()
        changed = False
        response = None

        if self.state == 'present':
            if old_response is not None:
                if self.type_properties_type is not None and self.type_properties_type != old_response['type_properties_type']:
                    self.fail("NAT type_properties_type cannot be changed.")
                else:
                    self.type_properties_type = old_response['type_properties_type']
                if self.mode is not None and self.mode != old_response['mode']:
                    self.fail("NAT mode cannot be changed.")
                else:
                    self.mode = old_response['mode']
                if self.ip_configuration_id is not None and self.ip_configuration_id != old_response['ip_configuration_id']:
                    changed = True
                else:
                    self.ip_configuration_id = old_response['ip_configuration_id']
                if self.internal_mappings is not None and old_response['internal_mappings'] != self.internal_mappings:
                    changed = True
                else:
                    self.internal_mappings = old_response['internal_mappings']

                if self.external_mappings is not None and self.external_mappings != old_response['external_mappings']:
                    changed = True
                else:
                    self.external_mappings = old_response['external_mappings']
            else:
                changed = True

            internal_mappings = None
            external_mappings = None
            if self.internal_mappings is not None:
                internal_mappings = [self.network_models.VpnNatRuleMapping(address_space=item) for item in self.internal_mappings]
            if self.external_mappings is not None:
                external_mappings = [self.network_models.VpnNatRuleMapping(address_space=item) for item in self.external_mappings]

            natrule_resource = self.network_models.VirtualNetworkGatewayNatRule(name=self.name,
                                                                                type_properties_type=self.type_properties_type,
                                                                                mode=self.mode,
                                                                                ip_configuration_id=self.ip_configuration_id,
                                                                                internal_mappings=internal_mappings,
                                                                                external_mappings=external_mappings)
            if changed:
                if not self.check_mode:
                    response = self.create_or_update_local_network_gateway(natrule_resource)
        else:
            if not self.check_mode:
                if old_response is not None:
                    self.delete_local_network_gateway()
                    changed = True
                    response = None
            else:
                changed = True

        if response is None:
            response = old_response
        self.results['state'] = response
        self.results['changed'] = changed
        return self.results

    def get_nat_rule(self):
        """Gets the specified nat rule"""
        response = None
        try:
            response = self.network_client.virtual_network_gateway_nat_rules.get(self.resource_group, self.virtual_network_gateway_name, self.name)
        except HttpResponseError as ec:
            self.log("Gets the specified local network gateway in a resource group Failed, Exception as {0}".format(ec))
            return None
        return self.format_response(response)

    def create_or_update_local_network_gateway(self, body):
        """Create or Update local network gateway"""
        response = None
        try:
            response = self.network_client.virtual_network_gateway_nat_rules.begin_create_or_update(self.resource_group,
                                                                                                    self.virtual_network_gateway_name, self.name, body)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except HttpResponseError as ec:
            self.fail("Create or Updated a local network gateway in a resource group Failed, Exception as {0}".format(ec))

        return self.format_response(response)

    def delete_local_network_gateway(self):
        """Deletes the specified local network gateway"""
        try:
            self.network_client.virtual_network_gateway_nat_rules.begin_delete(self.resource_group, self.virtual_network_gateway_name, self.name)
        except HttpResponseError as ec:
            self.fail("Deletes the specified nat rule, Exception as {0}".format(ec))
        return None

    def format_response(self, item):
        result = dict(
            resource_group=self.resource_group,
            id=item.id,
            name=item.name,
            type=item.type,
            etag=item.etag,
            provisioning_state=item.provisioning_state,
            type_properties_type=item.type_properties_type,
            mode=item.mode,
            internal_mappings=list(),
            external_mappings=list(),
            ip_configuration_id=item.ip_configuration_id
        )

        if item.internal_mappings is not None:
            for value in item.internal_mappings:
                result['internal_mappings'].append(value.address_space)
        if item.external_mappings is not None:
            for value in item.external_mappings:
                result['external_mappings'].append(value.address_space)
        return result


def main():
    """Main module execution code path"""

    AzureRMVirtualNetworkNatGateway()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_sase_sites_info
short_description: Information module for organizations _sase _sites
description:
  - Get all organizations _sase _sites.
  - List of enrolled SASE sites in this organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 10.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  search:
    description:
      - Search query parameter. If provided, filters results by search string.
    type: str
  status:
    description:
      - Status query parameter. If provided, filters results by site status label
        (e.g., 'good').
    type: str
  siteId:
    description:
      - SiteId query parameter. If provided, returns only the site matching this ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationSaseSites
    description: Complete reference of the getOrganizationSaseSites API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-sase-sites
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_sase_sites,
  - Paths used are
    get /organizations/{organizationId}/sase/sites,
"""

EXAMPLES = r"""
- name: Get all organizations _sase _sites
  cisco.meraki.organizations_sase_sites_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    search: string
    status: string
    siteId: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "siteId": "string",
          "network": {
            "id": "string"
          },
          "type": "string",
          "name": "string",
          "region": {
            "slug": "string"
          },
          "model": "string",
          "address": {
            "street": "string"
          },
          "vpn": {
            "type": "string"
          },
          "routing": {
            "defaultRoute": {
              "enabled": true
            }
          },
          "devices": {
            "primary": {
              "name": "string",
              "model": "string"
            },
            "spare": {
              "name": "string",
              "model": "string"
            }
          },
          "subnets": [
            {
              "subnet": "string"
            }
          ],
          "url": "string"
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



# Copyright (c) 2022, Håkon Lerring
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import copy
import json
import re
import typing as t
from http import HTTPStatus
from urllib import error as urllib_error
from urllib.parse import urlencode

from ansible.module_utils.urls import open_url

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule


def get_consul_url(configuration):
    return f"{configuration.scheme}://{configuration.host}:{configuration.port}/v1"


def get_auth_headers(configuration):
    if configuration.token is None:
        return {}
    else:
        return {"X-Consul-Token": configuration.token}


class RequestError(Exception):
    def __init__(self, status, response_data=None):
        self.status = status
        self.response_data = response_data

    def __str__(self):
        if self.response_data is None:
            # self.status is already the message (backwards compat)
            return self.status
        return f"HTTP {self.status}: {self.response_data}"


def handle_consul_response_error(response):
    if response.status_code >= HTTPStatus.BAD_REQUEST:  # 4xx and 5xx errors
        raise RequestError(f"{response.status_code} {response.content}")


AUTH_ARGUMENTS_SPEC = dict(
    host=dict(default="localhost"),
    port=dict(type="int", default=8500),
    scheme=dict(default="http"),
    validate_certs=dict(type="bool", default=True),
    token=dict(no_log=True),
    ca_path=dict(),
)


def camel_case_key(key):
    parts = []
    for part in key.split("_"):
        if part in {"id", "ttl", "jwks", "jwt", "oidc", "iam", "sts"}:
            parts.append(part.upper())
        else:
            parts.append(part.capitalize())
    return "".join(parts)


def validate_check(check):
    validate_duration_keys = ["Interval", "Ttl", "Timeout"]
    validate_tcp_regex = r"(?P<host>.*):(?P<port>(?:[0-9]+))$"
    if check.get("Tcp") is not None:
        match = re.match(validate_tcp_regex, check["Tcp"])
        if not match:
            raise Exception("tcp check must be in host:port format")
    for duration in validate_duration_keys:
        if duration in check and check[duration] is not None:
            check[duration] = validate_duration(check[duration])


def validate_duration(duration):
    if duration:
        if not re.search(r"\d+(?:ns|us|ms|s|m|h)", duration):
            duration = f"{duration}s"
    return duration


STATE_PARAMETER = "state"
STATE_PRESENT = "present"
STATE_ABSENT = "absent"

OPERATION_READ = "read"
OPERATION_CREATE = "create"
OPERATION_UPDATE = "update"
OPERATION_DELETE = "remove"


def _normalize_params(params, arg_spec):
    final_params = {}
    for k, v in params.items():
        if k not in arg_spec or v is None:  # Alias
            continue
        spec = arg_spec[k]
        if spec.get("type") == "list" and spec.get("elements") == "dict" and spec.get("options") and v:
            v = [_normalize_params(d, spec["options"]) for d in v]
        elif spec.get("type") == "dict" and spec.get("options") and v:
            v = _normalize_params(v, spec["options"])
        final_params[k] = v
    return final_params


class _ConsulModule:
    """Base class for Consul modules.

    This class is considered private, till the API is fully fleshed out.
    As such backwards incompatible changes can occur even in bugfix releases.
    """

    api_endpoint: str | None = None
    unique_identifiers: list | None = None
    result_key: str | None = None
    create_only_fields: set[str] = set()
    operational_attributes: set[str] = set()
    params: dict[str, t.Any] = {}

    def __init__(self, module: AnsibleModule) -> None:
        self._module = module
        self.params = _normalize_params(module.params, module.argument_spec)
        self.api_params = {
            k: camel_case_key(k) for k in self.params if k not in STATE_PARAMETER and k not in AUTH_ARGUMENTS_SPEC
        }

        self.operational_attributes.update({"CreateIndex", "CreateTime", "Hash", "ModifyIndex"})

    def execute(self):
        obj = self.read_object()

        changed = False
        diff = {}
        if self.params[STATE_PARAMETER] == STATE_PRESENT:
            obj_from_module = self.module_to_obj(obj is not None)
            if obj is None:
                operation = OPERATION_CREATE
                new_obj = self.create_object(obj_from_module)
                diff = {"before": {}, "after": new_obj}
                changed = True
            else:
                operation = OPERATION_UPDATE
                if self._needs_update(obj, obj_from_module):
                    new_obj = self.update_object(obj, obj_from_module)
                    diff = {"before": obj, "after": new_obj}
                    changed = True
                else:
                    new_obj = obj
        elif self.params[STATE_PARAMETER] == STATE_ABSENT:
            operation = OPERATION_DELETE
            if obj is not None:
                self.delete_object(obj)
                changed = True
                diff = {"before": obj, "after": {}}
            else:
                diff = {"before": {}, "after": {}}
            new_obj = None
        else:
            raise RuntimeError("Unknown state supplied.")

        result = {"changed": changed}
        if changed:
            result["operation"] = operation
            if self._module._diff:
                result["diff"] = diff
        if self.result_key:
            result[self.result_key] = new_obj
        self._module.exit_json(**result)

    def module_to_obj(self, is_update):
        obj = {}
        for k, v in self.params.items():
            result = self.map_param(k, v, is_update)
            if result:
                obj[result[0]] = result[1]
        return obj

    def map_param(self, k, v, is_update):
        def helper(item):
            return {camel_case_key(k): v for k, v in item.items()}

        def needs_camel_case(k):
            spec = self._module.argument_spec[k]
            return (spec.get("type") == "list" and spec.get("elements") == "dict" and spec.get("options")) or (
                spec.get("type") == "dict" and spec.get("options")
            )

        if k in self.api_params and v is not None:
            if isinstance(v, dict) and needs_camel_case(k):
                v = helper(v)
            elif isinstance(v, (list, tuple)) and needs_camel_case(k):
                v = [helper(i) for i in v]
            if is_update and k in self.create_only_fields:
                return
            return camel_case_key(k), v

    def _needs_update(self, api_obj, module_obj):
        api_obj = copy.deepcopy(api_obj)
        module_obj = copy.deepcopy(module_obj)
        return self.needs_update(api_obj, module_obj)

    def needs_update(self, api_obj, module_obj):
        for k, v in module_obj.items():
            if k not in api_obj:
                return True
            if api_obj[k] != v:
                return True
        return False

    def prepare_object(self, existing, obj):
        existing = {k: v for k, v in existing.items() if k not in self.operational_attributes}
        for k, v in obj.items():
            existing[k] = v
        return existing

    def id_from_obj(self, obj, camel_case=False):
        def key_func(key):
            return camel_case_key(key) if camel_case else key

        if self.unique_identifiers:
            for identifier in self.unique_identifiers:
                identifier = key_func(identifier)
                if identifier in obj:
                    return obj[identifier]
        return None

    def endpoint_url(self, operation, identifier=None):
        if operation == OPERATION_CREATE:
            return self.api_endpoint
        elif identifier:
            return f"{self.api_endpoint}/{identifier}"
        raise RuntimeError("invalid arguments passed")

    def read_object(self):
        identifier = self.id_from_obj(self.params)
        url = self.endpoint_url(OPERATION_READ, identifier)
        try:
            return self.get(url)
        except RequestError as e:
            if e.status == HTTPStatus.NOT_FOUND:
                return
            elif e.status == HTTPStatus.FORBIDDEN and b"ACL not found" in e.response_data:
                return
            raise

    def create_object(self, obj):
        if self._module.check_mode:
            return obj
        else:
            url = self.endpoint_url(OPERATION_CREATE)
            created_obj = self.put(url, data=self.prepare_object({}, obj))
            if created_obj is None:
                created_obj = self.read_object()
            return created_obj

    def update_object(self, existing, obj):
        merged_object = self.prepare_object(existing, obj)
        if self._module.check_mode:
            return merged_object
        else:
            url = self.endpoint_url(OPERATION_UPDATE, self.id_from_obj(existing, camel_case=True))
            updated_obj = self.put(url, data=merged_object)
            if updated_obj is None:
                updated_obj = self.read_object()
            return updated_obj

    def delete_object(self, obj):
        if self._module.check_mode:
            return {}
        else:
            url = self.endpoint_url(OPERATION_DELETE, self.id_from_obj(obj, camel_case=True))
            return self.delete(url)

    def _request(self, method, url_parts, data=None, params=None):
        module_params = self.params

        if not isinstance(url_parts, (tuple, list)):
            url_parts = [url_parts]
        if params:
            # Remove values that are None
            params = {k: v for k, v in params.items() if v is not None}

        ca_path = module_params.get("ca_path")
        base_url = f"{module_params['scheme']}://{module_params['host']}:{module_params['port']}/v1"
        url = "/".join([base_url] + list(url_parts))

        headers = {}
        token = self.params.get("token")
        if token:
            headers["X-Consul-Token"] = token

        try:
            if data is not None:
                data = json.dumps(data)
                headers["Content-Type"] = "application/json"
            if params:
                url = f"{url}?{urlencode(params)}"
            response = open_url(
                url,
                method=method,
                data=data,
                headers=headers,
                validate_certs=module_params["validate_certs"],
                ca_path=ca_path,
            )
            response_data = response.read()
            status = response.status if hasattr(response, "status") else response.getcode()

        except urllib_error.URLError as e:
            if isinstance(e, urllib_error.HTTPError):
                status = e.code
                response_data = e.fp.read()
            else:
                self._module.fail_json(
                    msg=f"Could not connect to consul agent at {module_params['host']}:{module_params['port']}, error was {e}"
                )
                raise

        if status >= HTTPStatus.BAD_REQUEST:  # 4xx and 5xx errors
            raise RequestError(status, response_data)

        if response_data:
            return json.loads(response_data)
        return None

    def get(self, url_parts, **kwargs):
        return self._request("GET", url_parts, **kwargs)

    def put(self, url_parts, **kwargs):
        return self._request("PUT", url_parts, **kwargs)

    def delete(self, url_parts, **kwargs):
        return self._request("DELETE", url_parts, **kwargs)
